==> jijzept-1.14.0rc2-cp39-cp39-manylinux_2_5_x86_64.manylinux1_x86_64.manylinux_2_28_x86_64.whl: jijzept/sampler/openjij/sa_cpu.py <==
from __future__ import annotations

import typing as typ

from dataclasses import dataclass
from numbers import Number

import numpy as np

from jijmodeling import Problem

from jijzept.entity.schema import SolverType
from jijzept.response import JijModelingResponse
from jijzept.sampler.base_sampler import (
    JijZeptBaseSampler,
    ParameterSearchParameters,
    merge_params_and_kwargs,
    sample_model,
)
from jijzept.sampler.sample_hubo import sample_hubo
from jijzept.sampler.sample_qubo import sample_qubo

T = typ.TypeVar("T")


@dataclass
class JijSAParameters:
    """Parameters for JijSASampler.

    Attributes:
        beta_min (Optional[float], optional): Minimum (initial) inverse temperature. If `None`, this will be set automatically.
        beta_max (Optional[float], optional): Maximum (final) inverse temperature. If `None`, this will be set automatically.
        num_sweeps (Optional[int], optional): The number of Monte-Carlo steps. If `None`, 1000 will be set.
        num_reads (Optional[int], optional): The number of samples. If `None`, 1 will be set.
        initial_state (Optional[dict], optional): Initial state. If `None`, this will be set automatically.
        updater (Optional[str], optional): Updater algorithm. "single spin flip" or "swendsen wang". If `None`, "single spin flip" will be set.
        sparse (Optional[bool], optional): If `True`, only non-zero matrix elements are stored, which will save memory. If `None`, `False` will be set.
        reinitialize_state (Optional[bool], optional): If `True`, reinitialize state for each run. If `None`, `True` will be set.
        seed (Optional[int], optional): Seed for Monte Carlo algorithm. If `None`, this will be set automatically.
    """

    beta_min: float | None = None
    beta_max: float | None = None
    num_sweeps: int | None = None
    num_reads: int | None = None
    initial_state: list | dict | None = None
    updater: str | None = None
    sparse: bool | None = None
    reinitialize_state: bool | None = None
    seed: int | None = None


class JijSASampler(JijZeptBaseSampler):
    """Simulated annealing sampler on CPU.

    This class is a sampler using SA that allows you to check if the mathematical model is correct in small problems.

    See [JijZeptBaseSampler][jijzept.sampler.base_sampler] for the constructor of the class.
    """

    solver_type = SolverType(queue_name="openjijsolver", solver="SA")
    hubo_solver_type = SolverType(queue_name="openjijsolver", solver="HUBOSA")
    jijmodeling_solver_type = SolverType(
        queue_name="openjijsolver", solver="SAParaSearch"
    )

    def sample_qubo(
        self,
        qubo: dict,
        constant: float = 0,
        parameters: JijSAParameters | None = None,
        timeout: int | None = None,
        sync: bool = True,
        queue_name: str | None = None,
        **kwargs,
    ) -> JijModelingResponse:
        """Sampling from QUBO.

        Args:
            qubo (dict[tuple[int, int], float]): QUBO dictionary.
            parameters (JijSAParameters | None, optional): SA parameters, defaults None.
            timeout (int | None, optional): The number of timeout [sec] for post request. If `None`, 3600 (one hour) will be set.
            sync (bool, optional): Synchronous mode.
            queue_name (str, optional): queue_name.
            **kwargs: You can pass SA parameters as keyword arguments instead of specifying them in `parameters`. See [JijSAParameters][jijzept.sampler.openjij.sa_cpu.JijSAParameters] for more information.

        Returns:
            An extended type of `jijmodleing.SampleSet` that stores sampling results.

        Examples:
            ``` python
            import jijzept as jz
            Q = {(0, 0): -1, (1, 1): -1, (2, 2): 1, (0, 1): -1, (1, 2): 1}
            sampler = jz.JijSASampler(config='config.toml')
            response = sampler.sample_qubo(Q)
            ```
        """
        param_dict = merge_params_and_kwargs(parameters, kwargs, JijSAParameters)

        if queue_name is None:
            queue_name = self.jijmodeling_solver_type.queue_name

        sample_set = sample_qubo(
            self.client,
            self.jijmodeling_solver_type.solver,
            queue_name=queue_name,
            qubo=qubo,
            constant=constant,
            timeout=timeout,
            sync=sync,
            **param_dict,
        )

        return sample_set

    def sample_hubo(
        self,
        J: dict,
        parameters: JijSAParameters | None = None,
        timeout: int | None = None,
        sync: bool = True,
        queue_name: str | None = None,
        **kwargs,
    ) -> JijModelingResponse:
        """Sampling from higher unconstrained binary optimization model (HUBO).

        Args:
            J (dict): Polynomial interactions.
            parameters: (JijSAParameters | None, optional): defaults None.
            timeout (Optional[int], optional): The number of timeout [sec] for post request. If `None`, 3600 (one hour) will be set.
            sync (bool, optional): Synchronous mode.
            queue_name (str, optional): queue_name.

        Returns:
            An extended type of `jijmodleing.SampleSet` that stores sampling results.

        Examples:
            ```python
            import jijzept as jz
            sampler = jz.JijSASampler(config='config.toml')
            J = {(0,): -1, (0, 1): -1, (0, 1, 2): 1}
            response = sampler.sample_hubo(J)
            ```
        """
        param_dict = merge_params_and_kwargs(parameters, kwargs, JijSAParameters)
        if queue_name is None:
            queue_name = self.jijmodeling_solver_type.queue_name

        sample_set = sample_hubo(
            self.client,
            self.jijmodeling_solver_type.solver,
            queue_name=queue_name,
            polynomial=J,
            constant=J.get((), 0.0),
            timeout=timeout,
            sync=sync,
            **param_dict,
        )
        return sample_set

    def sample_model(
        self,
        model: Problem,
        feed_dict: dict[str, Number | list | np.ndarray],
        multipliers: dict[str, Number] = {},
        fixed_variables: dict[str, dict[tuple[int, ...], int | float]] = {},
        search: bool = False,
        num_search: int = 15,
        algorithm: str | None = None,
        parameters: JijSAParameters | None = None,
        timeout: int | None = None,
        sync: bool = True,
        queue_name: str | None = None,
        **kwargs,
    ) -> JijModelingResponse:
        """Sample using JijModeling by means of the simulated annealing.

        Args:
            model (Expression): Mathematical expression of JijModeling.
            feed_dict (Dict[str, Union[Number, list, np.ndarray]]): The actual values to be assigned to the placeholders.
            multipliers (Dict[str, Number], optional): The actual multipliers for penalty terms, derived from constraint conditions.
            fixed_variables (Dict[str, Dict[Tuple[int, ...], Union[int, float]]]): dictionary of variables to fix.
            search (bool, optional): If `True`, the parameter search will be carried out, which tries to find better values of multipliers for penalty terms.
            num_search (int, optional): The number of parameter search iteration. Defaults to set 15. This option works if `search` is `True`.
            algorithm (Optional[str]): Algorithm for parameter search. Defaults to None.
            timeout (Optional[int], optional): The number of timeout [sec] for post request. If `None`, 3600 (one hour) will be set.
            sync (bool, optional): Synchronous mode.
            queue_name (str, optional): queue_name.

        Returns:
            JijModelingResponse: Stores minimum energy samples and other information.

        Examples:
            ```python
            import jijzept as jz
            import jijmodeling as jm
            n = jm.Placeholder('n')
            x = jm.Binary('x', shape=n)
            d = jm.Placeholder('d', shape=n)
            i = jm.Element("i", n)
            problem = jm.Problem('problem')
            problem += jm.Sum(i, d[i] * x[i])
            sampler = jz.JijSASampler(config='config.toml')
            response = sampler.sample_model(problem, feed_dict={'n': 5, 'd': [1,2,3,4,5]})
            ```
        """

        param_dict = merge_params_and_kwargs(parameters, kwargs, JijSAParameters)

        para_search_params = ParameterSearchParameters(
            multipliers=multipliers,
            mul_search=search,
            num_search=num_search,
            algorithm=algorithm,
        )

        if queue_name is None:
            queue_name = self.jijmodeling_solver_type.queue_name

        sample_set = sample_model(
            self.client,
            self.jijmodeling_solver_type.solver,
            queue_name=queue_name,
            problem=model,
            instance_data=feed_dict,
            fixed_variables=fixed_variables,
            parameter_search_parameters=para_search_params,
            timeout=timeout,
            sync=sync,
            **param_dict,
        )
        return sample_set

==> jijzept-1.14.1-cp39-cp39-manylinux_2_5_x86_64.manylinux1_x86_64.manylinux_2_28_x86_64.whl: jijzept/sampler/openjij/sa_cpu.py <==
from __future__ import annotations

import typing as typ

from dataclasses import dataclass
from numbers import Number

import numpy as np

from jijmodeling import Problem

from jijzept.entity.schema import SolverType
from jijzept.response import JijModelingResponse
from jijzept.sampler.base_sampler import (
    JijZeptBaseSampler,
    ParameterSearchParameters,
    merge_params_and_kwargs,
    sample_model,
)
from jijzept.sampler.sample_hubo import sample_hubo
from jijzept.sampler.sample_qubo import sample_qubo

T = typ.TypeVar("T")


@dataclass
class JijSAParameters:
    """Parameters for JijSASampler.

    Attributes:
        beta_min (Optional[float], optional): Minimum (initial) inverse temperature. If `None`, this will be set
        automatically.
        beta_max (Optional[float], optional): Maximum (final) inverse temperature. If `None`, this will be set
        automatically.
        num_sweeps (Optional[int], optional): The number of Monte-Carlo steps. If `None`, 1000 will be set.
        num_reads (Optional[int], optional): The number of samples. If `None`, 1 will be set.
        initial_state (Optional[dict], optional): Initial state. If `None`, this will be set automatically.
        updater (Optional[str], optional): Updater algorithm. "single spin flip" or "swendsen wang". If `None`,
        "single spin flip" will be set.
        sparse (Optional[bool], optional): If `True`, only non-zero matrix elements are stored, which will save
        memory. If `None`, `False` will be set.
        reinitialize_state (Optional[bool], optional): If `True`, reinitialize state for each run. If `None`,
        `True` will be set.
        seed (Optional[int], optional): Seed for Monte Carlo algorithm. If `None`, this will be set automatically.
    """

    beta_min: float | None = None
    beta_max: float | None = None
    num_sweeps: int | None = None
    num_reads: int | None = None
    initial_state: list | dict | None = None
    updater: str | None = None
    sparse: bool | None = None
    reinitialize_state: bool | None = None
    seed: int | None = None


class JijSASampler(JijZeptBaseSampler):
    """Simulated annealing sampler on CPU.

    This class is a sampler using SA that allows you to check if the mathematical model is correct in small problems.

    See [JijZeptBaseSampler][jijzept.sampler.base_sampler] for the constructor of the class.
    """

    solver_type = SolverType(queue_name="openjijsolver", solver="SA")
    hubo_solver_type = SolverType(queue_name="openjijsolver", solver="HUBOSA")
    jijmodeling_solver_type = SolverType(
        queue_name="openjijsolver", solver="SAParaSearch"
    )

    def sample_qubo(
        self,
        qubo: dict,
        constant: float = 0,
        parameters: JijSAParameters | None = None,
        max_wait_time: int | float | None = None,
        sync: bool = True,
        queue_name: str | None = None,
        **kwargs,
    ) -> JijModelingResponse:
        """Sampling from QUBO.

        To configure the solver, instantiate the `JijSAParameters` class and pass the instance to the `parameters` argument.

        Args:
            qubo (dict[tuple[int, int], float]): QUBO dictionary.
            parameters (JijSAParameters | None, optional): SA parameters, defaults None.
            max_wait_time (int | float | None, optional): The number of timeout [sec] for post request. If `None`, 3600 (one hour) will be set.
            Please note that this argument is for the `jijzept` timeout and not for configuring solver settings, such as solving time.
            sync (bool, optional): Synchronous mode.
            queue_name (str, optional): queue_name.
            **kwargs: You can pass SA parameters as keyword arguments instead of specifying them in `parameters`. See
            [JijSAParameters][jijzept.sampler.openjij.sa_cpu.JijSAParameters] for more information.

        Returns:
            An extended type of `jijmodleing.SampleSet` that stores sampling results.

        Examples:
            ``` python
            import jijzept as jz
            Q = {(0, 0): -1, (1, 1): -1, (2, 2): 1, (0, 1): -1, (1, 2): 1}
            sampler = jz.JijSASampler(config='config.toml')
            response = sampler.sample_qubo(Q)
            ```
        """
        param_dict = merge_params_and_kwargs(parameters, kwargs, JijSAParameters)

        if queue_name is None:
            queue_name = self.jijmodeling_solver_type.queue_name

        sample_set = sample_qubo(
            self.client,
            self.jijmodeling_solver_type.solver,
            queue_name=queue_name,
            qubo=qubo,
            constant=constant,
            max_wait_time=max_wait_time,
            sync=sync,
            **param_dict,
        )

        return sample_set

    def sample_hubo(
        self,
        J: dict,
        parameters: JijSAParameters | None = None,
        max_wait_time: int | float | None = None,
        sync: bool = True,
        queue_name: str | None = None,
        **kwargs,
    ) -> JijModelingResponse:
        """Sampling from higher unconstrained binary optimization model (HUBO).

        To configure the solver, instantiate the `JijSAParameters` class and pass the instance to the `parameters` argument.

        Args:
            J (dict): Polynomial interactions.
            parameters: (JijSAParameters | None, optional): defaults None.
            max_wait_time (int | float | None, optional): The number of timeout [sec] for post request.If `None`, 3600 (one hour) will be set.
            Please note that this argument is for the `jijzept` timeout and not for configuring solver settings, such as solving time.
            sync (bool, optional): Synchronous mode.
            queue_name (str, optional): queue_name.

        Returns:
            An extended type of `jijmodleing.SampleSet` that stores sampling results.

        Examples:
            ```python
            import jijzept as jz
            sampler = jz.JijSASampler(config='config.toml')
            J = {(0,): -1, (0, 1): -1, (0, 1, 2): 1}
            response = sampler.sample_hubo(J)
            ```
        """
        param_dict = merge_params_and_kwargs(parameters, kwargs, JijSAParameters)
        if queue_name is None:
            queue_name = self.jijmodeling_solver_type.queue_name

        sample_set = sample_hubo(
            self.client,
            self.jijmodeling_solver_type.solver,
            queue_name=queue_name,
            polynomial=J,
            constant=J.get((), 0.0),
            max_wait_time=max_wait_time,
            sync=sync,
            **param_dict,
        )
        return sample_set

    def sample_model(
        self,
        model: Problem,
        feed_dict: dict[str, Number | list | np.ndarray],
        multipliers: dict[str, Number] = {},
        fixed_variables: dict[str, dict[tuple[int, ...], int | float]] = {},
        search: bool = False,
        num_search: int = 15,
        algorithm: str | None = None,
        parameters: JijSAParameters | None = None,
        max_wait_time: int | float | None = None,
        sync: bool = True,
        queue_name: str | None = None,
        **kwargs,
    ) -> JijModelingResponse:
        """Sample using JijModeling by means of the simulated annealing.

        To configure the solver, instantiate the `JijSAParameters` class and pass the instance to the `parameters` argument.

        Args:
            model (Expression): Mathematical expression of JijModeling.
            feed_dict (Dict[str, Union[Number, list, np.ndarray]]): The actual values to be assigned to the
            placeholders.
            multipliers (Dict[str, Number], optional): The actual multipliers for penalty terms, derived from
            constraint conditions.
            fixed_variables (Dict[str, Dict[Tuple[int, ...], Union[int, float]]]): dictionary of variables to fix.
            search (bool, optional): If `True`, the parameter search will be carried out, which tries to find better
            values of multipliers for penalty terms.
            num_search (int, optional): The number of parameter search iteration. Defaults to set 15. This option
            works if `search` is `True`.
            algorithm (Optional[str]): Algorithm for parameter search. Defaults to None.
            max_wait_time (int | float | None, optional): The number of timeout [sec] for post request. If `None`, 3600 (one hour) will be set.
            Please note that this argument is for the `jijzept` timeout and not for configuring solver settings, such as solving time.
            sync (bool, optional): Synchronous mode.
            queue_name (str, optional): queue_name.

        Returns:
            JijModelingResponse: Stores minimum energy samples and other information.

        Examples:
            ```python
            import jijzept as jz
            import jijmodeling as jm
            n = jm.Placeholder('n')
            x = jm.Binary('x', shape=n)
            d = jm.Placeholder('d', shape=n)
            i = jm.Element("i", n)
            problem = jm.Problem('problem')
            problem += jm.Sum(i, d[i] * x[i])
            sampler = jz.JijSASampler(config='config.toml')
            response = sampler.sample_model(problem, feed_dict={'n': 5, 'd': [1,2,3,4,5]})
            ```
        """

        param_dict = merge_params_and_kwargs(parameters, kwargs, JijSAParameters)

        para_search_params = ParameterSearchParameters(
            multipliers=multipliers,
            mul_search=search,
            num_search=num_search,
            algorithm=algorithm,
        )

        if queue_name is None:
            queue_name = self.jijmodeling_solver_type.queue_name

        sample_set = sample_model(
            self.client,
            self.jijmodeling_solver_type.solver,
            queue_name=queue_name,
            problem=model,
            instance_data=feed_dict,
            fixed_variables=fixed_variables,
            parameter_search_parameters=para_search_params,
            max_wait_time=max_wait_time,
            sync=sync,
            **param_dict,
        )
        return sample_set
